<?php
/**
 * Created by PhpStorm.
 * User: benni
 * Date: 02.03.16
 * Time: 14:26
 */

namespace CMSExperts\Backendmess\Controller;


use Psr\Http\Message\ResponseInterface;
use Psr\Http\Message\ServerRequestInterface;
use TYPO3\CMS\Backend\Routing\UriBuilder;
use TYPO3\CMS\Backend\Utility\BackendUtility;
use TYPO3\CMS\Core\Utility\ExtensionManagementUtility;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Extbase\Mvc\View\ViewInterface;
use TYPO3\CMS\Fluid\View\StandaloneView;

class BackendController extends \TYPO3\CMS\Backend\Controller\BackendController
{
    protected $templatePath = 'EXT:backendmess/Resources/Private/Templates/';

    /**
     * Injects the request object for the current request or subrequest
     * As this controller goes only through the render() method, it is rather simple for now
     *
     * @param ServerRequestInterface $request the current request
     * @param ResponseInterface $response
     * @return ResponseInterface the response with the content
     */
    public function mainAction(ServerRequestInterface $request, ResponseInterface $response)
    {
        if (!$this->getBackendUser()->isAdmin()) {
            $this->render();
            $response->getBody()->write($this->content);
        } else {
            $response = $this->publicationOverviewAction($request, $response);
        }
        return $response;
    }


    /**
     * Show a list of all publications available to the user
     *
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @return ResponseInterface
     */
    public function publicationOverviewAction(ServerRequestInterface $request, ResponseInterface $response)
    {
        $view = $this->getView('PublicationOverview');
        $view->assign('publications', $this->getAvailablePublications());
        return $this->generateContent($view, $response);
    }

    /**
     * Show a list of all issues related to a publication
     *
     * @param ServerRequestInterface $request
     * @param ResponseInterface $response
     * @return ResponseInterface
     */
    public function issueOverviewAction(ServerRequestInterface $request, ResponseInterface $response)
    {
        $view = $this->getView('IssueOverview');
        $publicationUid = $request->getQueryParams()['id'];
        $issues = $this->getIssuesForPublication($publicationUid);
        $view->assign('issues', $issues);
        $view->assign('addArticleLink', $article);
        return $this->generateContent($view, $response);
    }


    protected function getAvailablePublications()
    {
        // get all pages with doktype "XYZ"
        $publications = [];
        $records = BackendUtility::getRecordsByField('pages', 'pid', '0', '', '', 'sorting');
        /** @var UriBuilder $uriBuilder */
        $uriBuilder = GeneralUtility::makeInstance(UriBuilder::class);
        foreach ($records as $record) {
            if ($this->getBackendUser()->isInWebMount($record['uid'])) {
                $record['issueOverviewUrl'] = $uriBuilder->buildUriFromRoute('publisher_issueoverview',
                    ['id' => $record['uid']]);
                $publications[$record['uid']] = $record;
            }
        }
        return $publications;
    }

    /**
     * 
     * @param int $publicationUid
     * @return array
     */
    protected function getIssuesForPublication($publicationUid) {
        $issues = [];
        $records = BackendUtility::getRecordsByField('pages', 'pid', $publicationUid, '', '', 'sorting');
        foreach ($records as $record) {
            if ($this->getBackendUser()->isInWebMount($record['uid'])) {
                $issues[$record['uid']] = $record;
            }
        }
        return $issues;

    }

    /**
     * @param string $templateName
     * @return \TYPO3\CMS\Fluid\View\StandaloneView
     */
    protected function getView($templateName)
    {
        $view = $this->getFluidTemplateObject($this->templatePath . 'Backend/' . ucfirst($templateName) . '.html');
        // Prepare the scaffolding, at this point extension may still add javascript and css

        // Render the TYPO3 logo in the left corner
        $logoUrl = $GLOBALS['TBE_STYLES']['logo'] ?: ExtensionManagementUtility::extRelPath('backend') . 'Resources/Public/Images/typo3-topbar@2x.png';
        $logoPath = GeneralUtility::resolveBackPath(PATH_typo3 . $logoUrl);
        list($logoWidth, $logoHeight) = @getimagesize($logoPath);

        // High-resolution?
        if (strpos($logoUrl, '@2x.') !== false) {
            $logoWidth = $logoWidth / 2;
            $logoHeight = $logoHeight / 2;
        }

        $view->assign('logoUrl', $logoUrl);
        $view->assign('logoWidth', $logoWidth);
        $view->assign('logoHeight', $logoHeight);
        $view->assign('logoLink', TYPO3_URL_GENERAL);
        $view->assign('siteName', $GLOBALS['TYPO3_CONF_VARS']['SYS']['sitename']);
        $view->assign('toolbar', $this->renderToolbar());
        return $view;
    }

    /**
     * @param StandaloneView $view
     * @param ResponseInterface $response
     * @return ResponseInterface
     */
    protected function generateContent(StandaloneView $view, ResponseInterface $response)
    {
        /******************************************************
         * Now put the complete backend document together
         ******************************************************/
        foreach ($this->cssFiles as $cssFileName => $cssFile) {
            $this->pageRenderer->addCssFile($cssFile);
            // Load additional css files to overwrite existing core styles
            if (!empty($GLOBALS['TBE_STYLES']['stylesheets'][$cssFileName])) {
                $this->pageRenderer->addCssFile($GLOBALS['TBE_STYLES']['stylesheets'][$cssFileName]);
            }
        }
        if (!empty($this->css)) {
            $this->pageRenderer->addCssInlineBlock('BackendInlineCSS', $this->css);
        }
        foreach ($this->jsFiles as $jsFile) {
            $this->pageRenderer->addJsFile($jsFile);
        }
        $this->generateJavascript();
        $this->pageRenderer->addJsInlineCode('BackendInlineJavascript', $this->js, false);
        $this->loadResourcesForRegisteredNavigationComponents();

        // Add state provider
        $this->getDocumentTemplate()->setExtDirectStateProvider();
        $states = $this->getBackendUser()->uc['BackendComponents']['States'];
        // Save states in BE_USER->uc
        $extOnReadyCode = '
			Ext.state.Manager.setProvider(new TYPO3.state.ExtDirectProvider({
				key: "BackendComponents.States",
				autoRead: false
			}));
		';

        if ($states) {
            $extOnReadyCode .= 'Ext.state.Manager.getProvider().initState(' . json_encode($states) . ');';
        }

        $extOnReadyCode .= '
			TYPO3.Backend = new TYPO3.Viewport(TYPO3.Viewport.configuration)';
        $this->pageRenderer->addExtOnReadyCode($extOnReadyCode);
        // Set document title:
        $title = $GLOBALS['TYPO3_CONF_VARS']['SYS']['sitename'] ? $GLOBALS['TYPO3_CONF_VARS']['SYS']['sitename'] . ' [TYPO3 CMS ' . TYPO3_version . ']' : 'TYPO3 CMS ' . TYPO3_version;
        // Renders the module page
        $this->content = $this->getDocumentTemplate()->render($title, $view->render());
        $response->getBody()->write($this->content);
        return $response;
    }
}